@@ -2,6 +2,7 @@
  * Copyright (C) 2013-2017 MulticoreWare, Inc
  *
  * Authors: Gopu Govindaswamy <gopu@multicorewareinc.com>
+ *          Ashok Kumar Mishra <ashok@multicorewareinc.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +28,31 @@
 
 using namespace X265_NS;
 
+bool PicQPAdaptationLayer::create(uint32_t width, uint32_t height, uint32_t partWidth, uint32_t partHeight, uint32_t numAQPartInWidthExt, uint32_t numAQPartInHeightExt)
+{
+    aqPartWidth = partWidth;
+    aqPartHeight = partHeight;
+    numAQPartInWidth = (width + partWidth - 1) / partWidth;
+    numAQPartInHeight = (height + partHeight - 1) / partHeight;
+
+    CHECKED_MALLOC_ZERO(dActivity, double, numAQPartInWidthExt * numAQPartInHeightExt);
+    CHECKED_MALLOC_ZERO(dQpOffset, double, numAQPartInWidthExt * numAQPartInHeightExt);
+    CHECKED_MALLOC_ZERO(dCuTreeOffset, double, numAQPartInWidthExt * numAQPartInHeightExt);
+
+    if (bQpSize)
+        CHECKED_MALLOC_ZERO(dCuTreeOffset8x8, double, numAQPartInWidthExt * numAQPartInHeightExt);
+
+    return true;
+fail:
+    return false;
+}
+
 bool Lowres::create(x265_param* param, PicYuv *origPic, uint32_t qgSize)
 {
     isLowres = true;
     bframes = param->bframes;
+    widthFullRes = origPic->m_picWidth;
+    heightFullRes = origPic->m_picHeight;
     width = origPic->m_picWidth / 2;
     lines = origPic->m_picHeight / 2;
     lumaStride = width + 2 * origPic->m_lumaMarginX;
@@ -49,7 +71,7 @@ bool Lowres::create(x265_param* param, PicYuv *origPic, uint32_t qgSize)
 
     size_t planesize = lumaStride * (lines + 2 * origPic->m_lumaMarginY);
     size_t padoffset = lumaStride * origPic->m_lumaMarginY + origPic->m_lumaMarginX;
-    if (!!param->rc.aqMode)
+    if (!!param->rc.aqMode || !!param->rc.hevcAq)
     {
         CHECKED_MALLOC_ZERO(qpAqOffset, double, cuCountFullRes);
         CHECKED_MALLOC_ZERO(invQscaleFactor, int, cuCountFullRes);
@@ -57,10 +79,50 @@ bool Lowres::create(x265_param* param, PicYuv *origPic, uint32_t qgSize)
         if (qgSize == 8)
             CHECKED_MALLOC_ZERO(invQscaleFactor8x8, int, cuCount);
     }
+
     if (origPic->m_param->bAQMotion)
         CHECKED_MALLOC_ZERO(qpAqMotionOffset, double, cuCountFullRes);
     if (origPic->m_param->bDynamicRefine)
         CHECKED_MALLOC_ZERO(blockVariance, uint32_t, cuCountFullRes);
+
+    if (!!param->rc.hevcAq)
+    {
+        m_maxCUSize = param->maxCUSize;
+        m_qgSize = qgSize;
+
+        uint32_t partWidth, partHeight, nAQPartInWidth, nAQPartInHeight;
+
+        pAQLayer = new PicQPAdaptationLayer[4];
+        maxAQDepth = 0;
+        for (uint32_t d = 0; d < 4; d++)
+        {
+            int ctuSizeIdx = 6 - g_log2Size[param->maxCUSize];
+            int aqDepth = g_log2Size[param->maxCUSize] - g_log2Size[qgSize];
+            if (!aqLayerDepth[ctuSizeIdx][aqDepth][d])
+                continue;
+
+            pAQLayer->minAQDepth = d;
+            partWidth = param->maxCUSize >> d;
+            partHeight = param->maxCUSize >> d;
+
+            if (minAQSize[ctuSizeIdx] == d)
+            {
+                pAQLayer[d].bQpSize = true;
+                nAQPartInWidth = maxBlocksInRow * 2;
+                nAQPartInHeight = maxBlocksInCol * 2;
+            }
+            else
+            {
+                pAQLayer[d].bQpSize = false;
+                nAQPartInWidth = (origPic->m_picWidth + partWidth - 1) / partWidth;
+                nAQPartInHeight = (origPic->m_picHeight + partHeight - 1) / partHeight;
+            }
+
+            maxAQDepth++;
+
+            pAQLayer[d].create(origPic->m_picWidth, origPic->m_picHeight, partWidth, partHeight, nAQPartInWidth, nAQPartInHeight);
+        }
+    }
     CHECKED_MALLOC(propagateCost, uint16_t, cuCount);
 
     /* allocate lowres buffers */
@@ -130,6 +192,25 @@ void Lowres::destroy()
     X265_FREE(invQscaleFactor8x8);
     X265_FREE(qpAqMotionOffset);
     X265_FREE(blockVariance);
+    if (maxAQDepth > 0)
+    {
+        for (uint32_t d = 0; d < 4; d++)
+        {
+            int ctuSizeIdx = 6 - g_log2Size[m_maxCUSize];
+            int aqDepth = g_log2Size[m_maxCUSize] - g_log2Size[m_qgSize];
+            if (!aqLayerDepth[ctuSizeIdx][aqDepth][d])
+                continue;
+
+            X265_FREE(pAQLayer[d].dActivity);
+            X265_FREE(pAQLayer[d].dQpOffset);
+            X265_FREE(pAQLayer[d].dCuTreeOffset);
+
+            if (pAQLayer[d].bQpSize == true)
+                X265_FREE(pAQLayer[d].dCuTreeOffset8x8);
+        }
+
+        delete[] pAQLayer;
+    }
 }
 // (re) initialize lowres state
 void Lowres::init(PicYuv *origPic, int poc)
