@@ -92,19 +92,14 @@ void DPB::recycleUnreferenced()
             m_freeList.pushBack(*curFrame);
             curFrame->m_encData->m_freeListNext = m_frameDataFreeList;
             m_frameDataFreeList = curFrame->m_encData;
-
-            if (curFrame->m_encData->m_meBuffer)
+            for (int i = 0; i < INTEGRAL_PLANE_NUM; i++)
             {
-                for (int i = 0; i < INTEGRAL_PLANE_NUM; i++)
+                if (curFrame->m_encData->m_meBuffer[i] != NULL)
                 {
-                    if (curFrame->m_encData->m_meBuffer[i] != NULL)
-                    {
-                        X265_FREE(curFrame->m_encData->m_meBuffer[i]);
-                        curFrame->m_encData->m_meBuffer[i] = NULL;
-                    }
+                    X265_FREE(curFrame->m_encData->m_meBuffer[i]);
+                    curFrame->m_encData->m_meBuffer[i] = NULL;
                 }
             }
-
             if (curFrame->m_ctuInfo != NULL)
             {
                 uint32_t widthInCU = (curFrame->m_param->sourceWidth + curFrame->m_param->maxCUSize - 1) >> curFrame->m_param->maxLog2CUSize;
@@ -136,9 +131,8 @@ void DPB::prepareEncode(Frame *newFrame)
     int pocCurr = slice->m_poc;
     int type = newFrame->m_lowres.sliceType;
     bool bIsKeyFrame = newFrame->m_lowres.bKeyframe;
-
     slice->m_nalUnitType = getNalUnitType(pocCurr, bIsKeyFrame);
-    if (slice->m_nalUnitType == NAL_UNIT_CODED_SLICE_IDR_W_RADL)
+    if (slice->m_nalUnitType == NAL_UNIT_CODED_SLICE_IDR_W_RADL || slice->m_nalUnitType == NAL_UNIT_CODED_SLICE_IDR_N_LP)
         m_lastIDR = pocCurr;
     slice->m_lastIDR = m_lastIDR;
     slice->m_sliceType = IS_X265_TYPE_B(type) ? B_SLICE : (type == X265_TYPE_P) ? P_SLICE : I_SLICE;
@@ -181,7 +175,10 @@ void DPB::prepareEncode(Frame *newFrame)
     // Mark pictures in m_piclist as unreferenced if they are not included in RPS
     applyReferencePictureSet(&slice->m_rps, pocCurr);
 
-    slice->m_numRefIdx[0] = X265_MIN(newFrame->m_param->maxNumReferences, slice->m_rps.numberOfNegativePictures); // Ensuring L0 contains just the -ve POC
+    if (slice->m_sliceType != I_SLICE)
+        slice->m_numRefIdx[0] = x265_clip3(1, newFrame->m_param->maxNumReferences, slice->m_rps.numberOfNegativePictures);
+    else
+        slice->m_numRefIdx[0] = X265_MIN(newFrame->m_param->maxNumReferences, slice->m_rps.numberOfNegativePictures); // Ensuring L0 contains just the -ve POC
     slice->m_numRefIdx[1] = X265_MIN(newFrame->m_param->bBPyramid ? 2 : 1, slice->m_rps.numberOfPositivePictures);
     slice->setRefPicList(m_picList);
 
@@ -230,11 +227,14 @@ void DPB::computeRPS(int curPoc, bool isRAP, RPS * rps, unsigned int maxDecPicBu
     {
         if ((iterPic->m_poc != curPoc) && iterPic->m_encData->m_bHasReferences)
         {
-            rps->poc[poci] = iterPic->m_poc;
-            rps->deltaPOC[poci] = rps->poc[poci] - curPoc;
-            (rps->deltaPOC[poci] < 0) ? numNeg++ : numPos++;
-            rps->bUsed[poci] = !isRAP;
-            poci++;
+            if ((m_lastIDR >= curPoc) || (m_lastIDR <= iterPic->m_poc))
+            {
+                    rps->poc[poci] = iterPic->m_poc;
+                    rps->deltaPOC[poci] = rps->poc[poci] - curPoc;
+                    (rps->deltaPOC[poci] < 0) ? numNeg++ : numPos++;
+                    rps->bUsed[poci] = !isRAP;
+                    poci++;
+            }
         }
         iterPic = iterPic->m_next;
     }
@@ -249,7 +249,7 @@ void DPB::computeRPS(int curPoc, bool isRAP, RPS * rps, unsigned int maxDecPicBu
 /* Marking reference pictures when an IDR/CRA is encountered. */
 void DPB::decodingRefreshMarking(int pocCurr, NalUnitType nalUnitType)
 {
-    if (nalUnitType == NAL_UNIT_CODED_SLICE_IDR_W_RADL)
+    if (nalUnitType == NAL_UNIT_CODED_SLICE_IDR_W_RADL || nalUnitType == NAL_UNIT_CODED_SLICE_IDR_N_LP)
     {
         /* If the nal_unit_type is IDR, all pictures in the reference picture
          * list are marked as "unused for reference" */
@@ -325,11 +325,9 @@ void DPB::applyReferencePictureSet(RPS *rps, int curPoc)
 NalUnitType DPB::getNalUnitType(int curPOC, bool bIsKeyFrame)
 {
     if (!curPOC)
-        return NAL_UNIT_CODED_SLICE_IDR_W_RADL;
-
+        return NAL_UNIT_CODED_SLICE_IDR_N_LP;
     if (bIsKeyFrame)
-        return m_bOpenGOP ? NAL_UNIT_CODED_SLICE_CRA : NAL_UNIT_CODED_SLICE_IDR_W_RADL;
-
+        return m_bOpenGOP ? NAL_UNIT_CODED_SLICE_CRA : m_bhasLeadingPicture ? NAL_UNIT_CODED_SLICE_IDR_W_RADL : NAL_UNIT_CODED_SLICE_IDR_N_LP;
     if (m_pocCRA && curPOC < m_pocCRA)
         // All leading pictures are being marked as TFD pictures here since
         // current encoder uses all reference pictures while encoding leading
